DEVICE := NRF51
DEVICESERIES := nrf51
DEVICE_VARIANT := xxaa

SDK_PATH = $(NORDIC_SDK_PATH)
SDK_INCLUDE_PATH = $(SDK_PATH)/Include/
SDK_SOURCE_PATH = $(SDK_PATH)/Source/
TEMPLATE_PATH += $(SDK_SOURCE_PATH)/templates/gcc/
OUTPUT_BINARY_DIRECTORY := $(OUTPUT_PATH)\_build

#ifeq ($(OS),Windows_NT)
include ./Makefile.windows
#else
#include $(TEMPLATE_PATH)Makefile.posix
#endif

#LINKER_SCRIPT = ./gcc_linker_script_nrf51.ld  

ifeq ($(LINKER_SCRIPT),)
	ifeq ($(USE_SOFTDEVICE), S110)
		ifeq ($(PROJECT_NAME), BARE)
			LINKER_SCRIPT = gcc_$(DEVICESERIES)_s110_$(DEVICE_VARIANT)_bare.ld
		else
			LINKER_SCRIPT = gcc_$(DEVICESERIES)_s110_$(DEVICE_VARIANT).ld
		endif
		OUTPUT_FILENAME := $(OUTPUT_FILENAME)_s110#_$(DEVICE_VARIANT)
		include Makefile.s110vars.mak
	else
		ifeq ($(USE_SOFTDEVICE), S210)
			LINKER_SCRIPT = gcc_$(DEVICESERIES)_s210_$(DEVICE_VARIANT).ld
			OUTPUT_FILENAME := $(OUTPUT_FILENAME)_s210#_$(DEVICE_VARIANT)
		else
			LINKER_SCRIPT = gcc_$(DEVICESERIES)_blank_$(DEVICE_VARIANT).ld
			OUTPUT_FILENAME := $(OUTPUT_FILENAME)#_$(DEVICE_VARIANT)
			include Makefile.stdvars.mak
		endif
	endif
else
# Use externally defined settings
endif

CPU := cortex-m0

# Toolchain commands
CC       		:= "$(GNU_INSTALL_ROOT)/bin/$(GNU_PREFIX)-gcc"
AS       		:= "$(GNU_INSTALL_ROOT)/bin/$(GNU_PREFIX)-as"
AR       		:= "$(GNU_INSTALL_ROOT)/bin/$(GNU_PREFIX)-ar" -r
LD       		:= "$(GNU_INSTALL_ROOT)/bin/$(GNU_PREFIX)-ld"
NM       		:= "$(GNU_INSTALL_ROOT)/bin/$(GNU_PREFIX)-nm"
OBJDUMP  		:= "$(GNU_INSTALL_ROOT)/bin/$(GNU_PREFIX)-objdump"
OBJCOPY  		:= "$(GNU_INSTALL_ROOT)/bin/$(GNU_PREFIX)-objcopy"
SIZE				:= "$(GNU_INSTALL_ROOT)/bin/$(GNU_PREFIX)-size"

SREC_CAT    := $(SREC_CAT_PATH)/srec_cat

MK 				:= mkdir
RM 				:= rm -rf

OBJECT_DIRECTORY := $(OUTPUT_PATH)\_build
LISTING_DIRECTORY := $(OUTPUT_PATH)\_build

C_SOURCE_FILES += system_$(DEVICESERIES).c
ASSEMBLER_SOURCE_FILES += gcc_startup_$(DEVICESERIES).s  # from new SDK

# Linker flags
LDFLAGS += -L"$(GNU_INSTALL_ROOT)/arm-none-eabi/lib/armv6-m"
LDFLAGS += -L"$(GNU_INSTALL_ROOT)/lib/gcc/arm-none-eabi/$(GNU_VERSION)/armv6-m"
LDFLAGS += -Xlinker -Map=$(LISTING_DIRECTORY)/$(OUTPUT_FILENAME).map
LDFLAGS += -mcpu=$(CPU) -mthumb -mabi=aapcs -L $(TEMPLATE_PATH) -T$(LINKER_SCRIPT)

# Compiler flags
CFLAGS += -mcpu=$(CPU) -mthumb -mabi=aapcs -D$(DEVICE) -D$(BOARD) -D$(TARGET_CHIP) --std=gnu99
CFLAGS += -Wall #-Werror
CFLAGS += -mfloat-abi=soft

# Assembler flags
ASMFLAGS += -x assembler-with-cpp

GLCD_INCLUDEPATHS += -I"glcd\inc"
GLCD_INCLUDEPATHS += -I"glcd\fonts"
GLCD_INCLUDEPATHS += -I"glcd\generic"

INCLUDEPATHS += $(GLCD_INCLUDEPATHS)
INCLUDEPATHS += -I"."
#INCLUDEPATHS += -I"../"
INCLUDEPATHS += -I"$(SDK_PATH)Include"
INCLUDEPATHS += -I"$(SDK_PATH)Include/gcc"
INCLUDEPATHS += -I"$(SDK_PATH)Include/ble"
INCLUDEPATHS += -I"$(SDK_PATH)Include/ble/softdevice"
INCLUDEPATHS += -I"$(SDK_PATH)Include/sd_common"
INCLUDEPATHS += -I"$(SDK_PATH)Include/app_common"
INCLUDEPATHS += -I"$(SDK_PATH)Include/ble/ble_services"
INCLUDEPATHS += -I"$(SDK_PATH)Include/ext_sensors"


# Sorting removes duplicates
BUILD_DIRECTORIES := $(sort $(OBJECT_DIRECTORY) $(OUTPUT_BINARY_DIRECTORY) $(LISTING_DIRECTORY) )
$(info build-directories is $(BUILD_DIRECTORIES))

####################################################################
# Rules                                                            #
####################################################################

C_SOURCE_FILENAMES = $(notdir $(C_SOURCE_FILES) )
ASSEMBLER_SOURCE_FILENAMES = $(notdir $(ASSEMBLER_SOURCE_FILES) )

# Make a list of source paths
C_SOURCE_PATHS += .
C_SOURCE_PATHS += ./fromSDK/
ASSEMBLER_SOURCE_PATHS += .
ASSEMBLER_SOURCE_PATHS += ./fromSDK

# TODO: Remove the below old paths from SDK sources
#       ALSO: Come up with a better way to organize files from the SDK.
#
#C_SOURCE_PATHS += ../ $(SDK_SOURCE_PATH) $(TEMPLATE_PATH) $(wildcard $(SDK_SOURCE_PATH)*/)  $(wildcard $(SDK_SOURCE_PATH)ext_sensors/*/) $(wildcard $(SDK_SOURCE_PATH)ble/*/)
#ASSEMBLER_SOURCE_PATHS = ../ $(SDK_SOURCE_PATH) $(TEMPLATE_PATH) $(wildcard $(SDK_SOURCE_PATH)*/)
#
#C_OBJECTS = $(addprefix $(OBJECT_DIRECTORY)/, $(C_SOURCE_FILENAMES:.c=.o) )
#ASSEMBLER_OBJECTS = $(addprefix $(OBJECT_DIRECTORY)/, $(ASSEMBLER_SOURCE_FILENAMES:.s=.o) )

C_OBJECTS = $(addprefix $(OBJECT_DIRECTORY)/, $(C_SOURCE_FILENAMES:.c=.o) )
ASSEMBLER_OBJECTS = $(addprefix $(OBJECT_DIRECTORY)/, $(ASSEMBLER_SOURCE_FILENAMES:.s=.o) )

# Set source lookup paths
vpath %.c $(C_SOURCE_PATHS)
vpath %.s $(ASSEMBLER_SOURCE_PATHS)

# Include automatically previously generated dependencies
#-include $(addprefix $(OBJECT_DIRECTORY)/, $(C_OBJECTS:.o=.d))
-include $(C_OBJECTS:.o=.d)

HEX = $(OUTPUT_BINARY_DIRECTORY)\$(OUTPUT_FILENAME).$(IMAGEID).$(VERSION).hex
$(info hex is $(HEX))
HEXCONFIG = $(OUTPUT_BINARY_DIRECTORY)\$(OUTPUT_FILENAME).cfg.hex
$(info hexconfig is $(HEXCONFIG))
HEXWITHSUM = $(OUTPUT_BINARY_DIRECTORY)\$(OUTPUT_FILENAME).$(IMAGEID).$(VERSION).sum.hex
HEXICAST = $(OUTPUT_BINARY_DIRECTORY)\$(OUTPUT_FILENAME).$(IMAGEID).$(VERSION).icast.hex
METERICAST = $(OUTPUT_BINARY_DIRECTORY)\$(METERFW).$(METERIMAGE).$(METERVERSION).icast.hex
HEXWITHCFG = $(OUTPUT_BINARY_DIRECTORY)\$(OUTPUT_FILENAME).$(IMAGEID).$(VERSION).all.hex
ELF = $(OUTPUT_BINARY_DIRECTORY)\$(OUTPUT_FILENAME).$(IMAGEID).$(VERSION).elf
BIN = $(OUTPUT_BINARY_DIRECTORY)\$(OUTPUT_FILENAME).$(IMAGEID).$(VERSION).bin
OUT = $(OUTPUT_BINARY_DIRECTORY)/$(OUTPUT_FILENAME).out

TARGETS = $(HEX) $(HEXWITHSUM) $(HEXWITHCFG) $(HEXICAST) $(METERICAST) $(BIN) $(OUT)

$(info targets is $(TARGETS))

### Targets
debug:    CFLAGS += -DDEBUG -g3 -O0
debug:    ASMFLAGS += -DDEBUG -g3 -O0
debug:		$(TARGETS)
#debug:    $(OUTPUT_BINARY_DIRECTORY)/$(OUTPUT_FILENAME).bin $(OUTPUT_BINARY_DIRECTORY)/$(OUTPUT_FILENAME).hex

.PHONY: release
release: #clean
release:  CFLAGS += -DNDEBUG -Os #-O3
release:  ASMFLAGS += -DNDEBUG -Os #-O3
release:	$(TARGETS)
#release:  $(OUTPUT_BINARY_DIRECTORY)/$(OUTPUT_FILENAME).bin $(OUTPUT_BINARY_DIRECTORY)/$(OUTPUT_FILENAME).hex

echostuff:
	@echo C_OBJECTS: [$(C_OBJECTS)]
	@echo C_SOURCE_FILES: [$(C_SOURCE_FILES)]

## Create build directories
$(BUILD_DIRECTORIES):
	$(MK) $@

## Create objects from C source files
$(OBJECT_DIRECTORY)/%.o: %.c
# Build header dependencies
	$(CC) $(CFLAGS) $(INCLUDEPATHS) -M $< -MF "$(@:.o=.d)" -MT $@
# Do the actual compilation
	$(CC) $(CFLAGS) $(INCLUDEPATHS) -c -o $@ $<

## Assemble .s files
$(OBJECT_DIRECTORY)/%.o: %.s
	$(CC) $(ASMFLAGS) $(INCLUDEPATHS) -c -o $@ $<

$(HEX): $(BUILD_DIRECTORIES) $(C_OBJECTS) $(ASSEMBLER_OBJECTS) $(LIBRARIES)
	$(CC) $(LDFLAGS) $(C_OBJECTS) $(ASSEMBLER_OBJECTS) $(LIBRARIES) glcd.a -o $(ELF) -lm
	$(OBJCOPY) -Oihex $(ELF) $(HEX)
	$(SIZE) $(ELF)

$(HEXWITHSUM): $(HEX)
	$(SREC_CAT) $(HEX) -intel \
		-fill 0xff $(MAIN_CODE_START) $(IMAGEID_START) \
		-fill $(IMAGEID) $(IMAGEID_START) $(VERSION_START) \
		-fill $(VERSION) $(VERSION_START) $(IMAGECRC_START) \
		-crc16-l-e $(IMAGECRC_START) -XMODEM \
		-o $(HEXWITHSUM) -intel

#$(HEXWITHSUM): $(HEX)
#	$(SREC_CAT) $(HEX) -intel \
#		-fill 0xff 0x00000000 0x0001effc \
#		-fill $(IMAGEID) 0x0001effc 0x0001effd \
#		-fill $(VERSION) 0x0001effd 0x0001effe \
#		-crc16-l-e 0x0001effe -XMODEM \
#		-o $(HEXWITHSUM) -intel

$(BIN): $(HEXWITHSUM)
	$(SREC_CAT) \
		$(HEXWITHSUM) -intel \
		-o $(BIN) -binary

$(HEXCONFIG): $(HEX) $(HEXCONFIG_MAKEFILE)
	$(HEXCONFIG_RECIPE)

$(METERICAST): $(METERICAST_DEP)
	$(METERICAST_RECIPE)

$(HEXWITHCFG): $(HEXWITHSUM) $(HEXCONFIG)
	$(SREC_CAT) \
		$(HEXWITHSUM) -intel \
		$(HEXCONFIG) -intel \
		-o $(HEXWITHCFG) -intel


$(info ICAST_BUFFER = $(ICAST_BUFFER))

# create an icast image - generate a filled map (all 0s) and shift the 
# code to 0x00020000
$(HEXICAST): $(HEXWITHSUM)
	$(SREC_CAT) $(HEXWITHSUM) -intel \
		-offset $(ICAST_BUFFER) \
		-fill 0x00 $(ICAST_MAP) $(ICAST_MAP_ZEROS_END) \
		-fill 0xff $(ICAST_MAP_ZEROS_END) $(ICAST_MAP_END) \
		-fill 0x00 $(ICAST_STATUS) $(ICAST_STATUS_ZEROS_END) \
		-fill $(IMAGEID) $(ICAST_IMAGEID) $(ICAST_IMAGEID_END) \
		-fill 0x00 $(ICAST_IMAGEID_END) $(ICAST_VERSION) \
		-fill $(VERSION) $(ICAST_VERSION) $(ICAST_VERSION_END) \
		-fill 0x00 $(ICAST_VERSION_END) $(ICAST_VERSION_ZEROS_END) \
		-fill 0xff $(ICAST_VERSION_ZEROS_END) $(ICAST_STATUS_END) \
		-o $(HEXICAST) -intel

#$(HEXICAST): $(HEXWITHSUM)
#	$(SREC_CAT) $(HEXWITHSUM) -intel \
#		-offset $(ICAST_BUFFER) \
#		-fill 0x00 $(ICAST_MAP) 0x0001f3e0 \
#		-fill 0xff 0x0001f3e0 $(ICAST_MAP_END) \
#		-fill 0x00 0x0001f800 0x0001f804 \
#		-fill $(IMAGEID) 0x0001f804 0x0001f805 \
#		-fill 0x00 0x0001f805 0x0001f808 \
#		-fill $(VERSION) 0x0001f808 0x0001f809 \
#		-fill 0x00 0x0001f809 0x0001f80c \
#		-fill 0xff 0x0001f80c $(ICAST_STATUS_END) \
#		-o $(HEXICAST) -intel



## Link C and assembler objects to an .out file
$(OUT): $(BUILD_DIRECTORIES) $(C_OBJECTS) $(ASSEMBLER_OBJECTS) $(LIBRARIES)
	$(CC) $(LDFLAGS) $(C_OBJECTS) $(ASSEMBLER_OBJECTS) $(LIBRARIES) -o $(OUTPUT_BINARY_DIRECTORY)/$(OUTPUT_FILENAME).out -lm

## Create binary .bin file from the .out file
$(OUTPUT_BINARY_DIRECTORY)/$(OUTPUT_FILENAME).bin: $(OUTPUT_BINARY_DIRECTORY)/$(OUTPUT_FILENAME).out
	$(OBJCOPY) -O binary $(OUTPUT_BINARY_DIRECTORY)/$(OUTPUT_FILENAME).out $(OUTPUT_BINARY_DIRECTORY)/$(OUTPUT_FILENAME).bin

## Create binary .hex file from the .out file
$(OUTPUT_BINARY_DIRECTORY)/$(OUTPUT_FILENAME).hex: $(OUTPUT_BINARY_DIRECTORY)/$(OUTPUT_FILENAME).out
	$(OBJCOPY) -O ihex $(OUTPUT_BINARY_DIRECTORY)/$(OUTPUT_FILENAME).out $(OUTPUT_BINARY_DIRECTORY)/$(OUTPUT_FILENAME).hex

## Default build target
.PHONY: all
all: clean debug

clean:
	$(RM) $(OUTPUT_BINARY_DIRECTORY)

## Program device
.PHONY: flash
flash: $(OUTPUT_BINARY_DIRECTORY)/$(OUTPUT_FILENAME).hex
	nrfjprog --reset --program $(OUTPUT_BINARY_DIRECTORY)/$(OUTPUT_FILENAME).hex

.PHONY: reset
reset:
	nrfjprog --reset

.PHONY: erase
erase:
	nrfjprog --erase

